// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

glib::wrapper! {
    #[doc(alias = "GdkWin32Drag")]
    pub struct Win32Drag(Object<ffi::GdkWin32Drag, ffi::GdkWin32DragClass>) @extends gdk::Drag;

    match fn {
        type_ => || ffi::gdk_win32_drag_get_type(),
    }
}

impl Win32Drag {}
