// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{IMContext, InputHints, InputPurpose};
use glib::{prelude::*, translate::*};

glib::wrapper! {
    #[doc(alias = "GtkIMMulticontext")]
    pub struct IMMulticontext(Object<ffi::GtkIMMulticontext, ffi::GtkIMMulticontextClass>) @extends IMContext;

    match fn {
        type_ => || ffi::gtk_im_multicontext_get_type(),
    }
}

impl IMMulticontext {
    pub const NONE: Option<&'static IMMulticontext> = None;

    #[doc(alias = "gtk_im_multicontext_new")]
    pub fn new() -> IMMulticontext {
        assert_initialized_main_thread!();
        unsafe { IMContext::from_glib_full(ffi::gtk_im_multicontext_new()).unsafe_cast() }
    }

    // rustdoc-stripper-ignore-next
    /// Creates a new builder-pattern struct instance to construct [`IMMulticontext`] objects.
    ///
    /// This method returns an instance of [`IMMulticontextBuilder`](crate::builders::IMMulticontextBuilder) which can be used to create [`IMMulticontext`] objects.
    pub fn builder() -> IMMulticontextBuilder {
        IMMulticontextBuilder::new()
    }
}

impl Default for IMMulticontext {
    fn default() -> Self {
        Self::new()
    }
}

// rustdoc-stripper-ignore-next
/// A [builder-pattern] type to construct [`IMMulticontext`] objects.
///
/// [builder-pattern]: https://doc.rust-lang.org/1.0.0/style/ownership/builders.html
#[must_use = "The builder must be built to be used"]
pub struct IMMulticontextBuilder {
    builder: glib::object::ObjectBuilder<'static, IMMulticontext>,
}

impl IMMulticontextBuilder {
    fn new() -> Self {
        Self {
            builder: glib::object::Object::builder(),
        }
    }

    pub fn input_hints(self, input_hints: InputHints) -> Self {
        Self {
            builder: self.builder.property("input-hints", input_hints),
        }
    }

    pub fn input_purpose(self, input_purpose: InputPurpose) -> Self {
        Self {
            builder: self.builder.property("input-purpose", input_purpose),
        }
    }

    // rustdoc-stripper-ignore-next
    /// Build the [`IMMulticontext`].
    #[must_use = "Building the object from the builder is usually expensive and is not expected to have side effects"]
    pub fn build(self) -> IMMulticontext {
        self.builder.build()
    }
}

mod sealed {
    pub trait Sealed {}
    impl<T: super::IsA<super::IMMulticontext>> Sealed for T {}
}

pub trait IMMulticontextExt: IsA<IMMulticontext> + sealed::Sealed + 'static {
    #[doc(alias = "gtk_im_multicontext_get_context_id")]
    #[doc(alias = "get_context_id")]
    fn context_id(&self) -> glib::GString {
        unsafe {
            from_glib_none(ffi::gtk_im_multicontext_get_context_id(
                self.as_ref().to_glib_none().0,
            ))
        }
    }

    #[doc(alias = "gtk_im_multicontext_set_context_id")]
    fn set_context_id(&self, context_id: Option<&str>) {
        unsafe {
            ffi::gtk_im_multicontext_set_context_id(
                self.as_ref().to_glib_none().0,
                context_id.to_glib_none().0,
            );
        }
    }
}

impl<O: IsA<IMMulticontext>> IMMulticontextExt for O {}
