// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

glib::wrapper! {
    #[doc(alias = "GdkSnapshot")]
    pub struct Snapshot(Object<ffi::GdkSnapshot, ffi::GdkSnapshotClass>);

    match fn {
        type_ => || ffi::gdk_snapshot_get_type(),
    }
}

impl Snapshot {
    pub const NONE: Option<&'static Snapshot> = None;
}
